#include "animated_polygon.h"
using namespace std;


namespace ist {
namespace i3d {


struct AVTXHeader {
	unsigned int version_number;
	unsigned int index_count;
	unsigned int vertex_count;
	unsigned int vertex_clip_count;
};

struct AVTXClipHeader {
	unsigned int frame_count;
	unsigned int label_length;
};




BAnimatedModel::BAnimatedModel() :
	num_vertex(0)
{}

BAnimatedModel::~BAnimatedModel()
{}

void BAnimatedModel::draw(const string& clipname, float t) const
{
	draw(vtx_clip.find(clipname), t);
}

void BAnimatedModel::draw(iterator iter, float t) const
{}

void BAnimatedModel::setup()
{
	if(vtx_clip.empty() || vtx_clip.begin()->second.empty())
		return;

	vertex_cont& vc = vtx_clip.begin()->second.begin()->second;
	bbox.min = vc.front();
	bbox.max = vc.front();
	for(vertex_cont::iterator p=vc.begin(); p!=vc.end(); ++p) {
		if(p->x > bbox.max.x)      bbox.max.x = p->x;
		else if(p->x < bbox.min.x) bbox.min.x = p->x;
		if(p->y > bbox.max.y)      bbox.max.y = p->y;
		else if(p->y < bbox.min.y) bbox.min.y = p->y;
		if(p->z > bbox.max.z)      bbox.max.z = p->z;
		else if(p->z < bbox.min.z) bbox.min.z = p->z;
	}

	vector4 center = (bbox.max+bbox.min)/2;
	bsphere.pos = center;
	bsphere.radius = (bbox.max-center).norm();
}



bool BAnimatedModel::load(const string& filename) {
	clear();

	if(strncmp(&filename[filename.size()]-4, ".avx", 5)!=0)
		return false;

	FILE *in = fopen(filename.c_str(), "rb");
	if(in==0) return false;

	AVTXHeader header;
	fread(&header.version_number, sizeof(int), 1, in);
	fread(&header.index_count, sizeof(int), 1, in);
	fread(&header.vertex_count, sizeof(int), 1, in);
	fread(&header.vertex_clip_count, sizeof(int), 1, in);
	if(header.version_number!=100)
		return false;

	idx_array.resize(header.index_count);
	tex_array.resize(header.index_count);
	num_vertex = header.vertex_count;


	for(unsigned int i=0; i<idx_array.size(); ++i)
		fread(&idx_array[i], sizeof(int), 1, in);

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		fread(&tex_array[i].x, sizeof(float), 1, in);
		fread(&tex_array[i].y, sizeof(float), 1, in);
	}

	for(unsigned int i=0; i<header.vertex_clip_count; ++i) {
		AVTXClipHeader clip_header;
		vector<char> temp_string(256, '\0');
		fread(&clip_header.frame_count, sizeof(int), 1, in);
		fread(&clip_header.label_length, sizeof(int), 1, in);
		fread(&temp_string[0], sizeof(char), clip_header.label_length, in);

		vertex_sequence& seq = vtx_clip[&temp_string[0]];

		for(unsigned int j=0; j<clip_header.frame_count; ++j) {
			float t;
			fread(&t, sizeof(float), 1, in);

			vertex_cont& vcont = seq[t];
			vcont.resize(header.vertex_count);

			for(unsigned int k=0; k<vcont.size(); ++k) {
				fread(&vcont[k].x, sizeof(float), 1, in);
				fread(&vcont[k].y, sizeof(float), 1, in);
				fread(&vcont[k].z, sizeof(float), 1, in);
			}
		}
	}

	fclose(in);

	setup();

	return true;
}

bool BAnimatedModel::save(const string& filename) {
	FILE *out = fopen(filename.c_str(), "wb");
	if(out==0) return false;

	AVTXHeader header={
		100,
		idx_array.size(),
		num_vertex,
		vtx_clip.size()
	};

	fwrite(&header.version_number, sizeof(int), 1, out);
	fwrite(&header.index_count, sizeof(int), 1, out);
	fwrite(&header.vertex_count, sizeof(int), 1, out);
	fwrite(&header.vertex_clip_count, sizeof(int), 1, out);

	for(unsigned int i=0; i<idx_array.size(); ++i)
		fwrite(&idx_array[i], sizeof(int), 1, out);

	for(unsigned int i=0; i<tex_array.size(); ++i) {
		fwrite(&tex_array[i].x, sizeof(float), 1, out);
		fwrite(&tex_array[i].y, sizeof(float), 1, out);
	}

	for(vertex_clip::iterator p=vtx_clip.begin(); p!=vtx_clip.end(); ++p) {
		vertex_sequence& seq = p->second;
		AVTXClipHeader clip_header={
			seq.size(),
			p->first.size(),
		};

		fwrite(&clip_header.frame_count, sizeof(int), 1, out);
		fwrite(&clip_header.label_length, sizeof(int), 1, out);
		fwrite(p->first.c_str(), sizeof(char), p->first.size(), out);

		for(vertex_sequence::iterator q=seq.begin(); q!=seq.end(); ++q) {
			fwrite(&q->first, sizeof(float), 1, out);

			vertex_cont& vcont = q->second;
			for(unsigned int i=0; i<vcont.size(); ++i) {
				fwrite(&vcont[i].x, sizeof(float), 1, out);
				fwrite(&vcont[i].y, sizeof(float), 1, out);
				fwrite(&vcont[i].z, sizeof(float), 1, out);
			}
		}
	}

	fclose(out);

	return true;
}

void BAnimatedModel::clear() {
	vtx_clip.clear();
	tex_array.clear();
	idx_array.clear();
	num_vertex=0;
}

void BAnimatedModel::vertex_sequence::morph(float t, vertex_cont& t_vertex) const
{
	if(t<=beginFrame()) {
		t_vertex = begin()->second;
	}
	else if(t>=endFrame()) {
		t_vertex = (--end())->second;
	}
	else {
		vertex_sequence::range_t range = equal_range(t);
		if(range.first->first==t) {
			t_vertex = range.first->second;
		}
		else {
			--range.first;
			const float& keyframe1 = range.first->first;
			const float& keyframe2 = range.second->first;
			const vertex_cont& vcont1 = range.first->second;
			const vertex_cont& vcont2 = range.second->second;
			t_vertex.resize(vcont1.size());
			for(size_t i=0; i<vcont1.size(); ++i) {
				vector3 dir = (vcont2[i]-vcont1[i])/(keyframe2-keyframe1);
				t_vertex[i] = vcont1[i] + dir*(t-keyframe1);
			}
		}
	}
}




BCachedAnimatedModel::BCachedAnimatedModel() :
	num_vertex(0),
	facet(60.0f),
	enable_normal(true)
{}

BCachedAnimatedModel::BCachedAnimatedModel(const std::string& filename, bool en) :
	num_vertex(0),
	facet(60.0f),
	enable_normal(true)
{
	enableNormal(en);
	load(filename);
}

BCachedAnimatedModel::~BCachedAnimatedModel()
{}

bool BCachedAnimatedModel::load(const string& filename) {
	BAnimatedModel tpolygon;
	if(!tpolygon.load(filename))
		return false;

	tex_array = tpolygon.tex_array;
	idx_array = tpolygon.idx_array;
	num_vertex = tpolygon.num_vertex;

	vertex_cont t_vertex;
	typedef BAnimatedModel::vertex_clip::iterator clip_iterator;
	for(clip_iterator p=tpolygon.vtx_clip.begin(); p!=tpolygon.vtx_clip.end(); ++p) {
		vertex_sequence& seq = vtx_clip[p->first];
		seq.begin_frame = tpolygon.beginFrame(p->first);
		seq.end_frame = tpolygon.endFrame(p->first);
		int frame_length = seq.endFrame()-seq.beginFrame();
		seq.vertex.resize(std::max<int>(1, frame_length));
		seq.normal.resize(std::max<int>(1, frame_length));
		int t=0;
		do {
			p->second.morph(t+seq.beginFrame(), t_vertex);
			setVertex(t_vertex, seq.vertex[t]);
			if(enable_normal)
				setNormal(t_vertex, seq.normal[t]);
			++t;
		} while(t<frame_length);
	}

	setup();
	return true;
}

void BCachedAnimatedModel::setVertex(vertex_cont& vtx_array, vertex_cont& vvtx_array)
{
	vvtx_array.resize(idx_array.size());
	for(size_t i=0; i<idx_array.size(); ++i) {
		vvtx_array[i] = vtx_array[idx_array[i]];
	}
}

void BCachedAnimatedModel::setNormal(vertex_cont& vtx_array, normal_cont& vnml_array)
{
	normal_cont nml_array;

	for(size_t i=0; i<idx_array.size()/3; ++i) {
		vector3 vec1 = vtx_array[ idx_array[i*3+1] ] - vtx_array[ idx_array[i*3] ];
		vector3 vec2 = vtx_array[ idx_array[i*3+2] ] - vtx_array[ idx_array[i*3] ];
		nml_array.push_back(vector3().cross(vec1, vec2));
	}

	vnml_array.reserve(idx_array.size());
	for(size_t i=0; i<idx_array.size(); ++i) {
		int union_num = 0;
		vector3 std_normal = nml_array[i/3];
		vector3 cur_normal = std_normal;

		for(size_t j=0; j<idx_array.size()/3; ++j) {
			if(idx_array[i]==idx_array[j*3+0] || idx_array[i]==idx_array[j*3+1] || idx_array[i]==idx_array[j*3+2]) {
				if(std_normal!=nml_array[j] && std_normal.cos(nml_array[j]) <= facet*radian) {
					cur_normal+=nml_array[j];
					union_num++;
				}
			}
		}

		cur_normal.normalize();
		vnml_array.push_back(cur_normal);
	}
}

void BCachedAnimatedModel::draw(const string& clipname, int t) const
{
	draw(vtx_clip.find(clipname), t);
}

void BCachedAnimatedModel::draw(iterator p, int t) const
{}

void BCachedAnimatedModel::setup()
{
	if(vtx_clip.empty())
		return;

	vertex_cont& vtx_array = vtx_clip.begin()->second.vertex.front();
	bbox.min = vtx_array.front();
	bbox.max = vtx_array.front();
	for(vertex_cont::iterator p=vtx_array.begin(); p!=vtx_array.end(); ++p) {
		if(p->x > bbox.max.x)      bbox.max.x = p->x;
		else if(p->x < bbox.min.x) bbox.min.x = p->x;
		if(p->y > bbox.max.y)      bbox.max.y = p->y;
		else if(p->y < bbox.min.y) bbox.min.y = p->y;
		if(p->z > bbox.max.z)      bbox.max.z = p->z;
		else if(p->z < bbox.min.z) bbox.min.z = p->z;
	}

	vector4 center = (bbox.max+bbox.min)/2;
	bsphere.pos = center;
	bsphere.radius = (bbox.max-center).norm();
}

void BCachedAnimatedModel::clear() {
	vtx_clip.clear();
	tex_array.clear();
	idx_array.clear();
	num_vertex=0;
}


} //i3d
} //ist
