#ifndef IST_BITMAP
#define IST_BITMAP

#include <vector>
#include <string>
#include "rgba.h"



namespace ist {


class Bitmap {
private:
	unsigned int _width, _height;
	std::vector<bRGBA> data;

public:
	typedef std::vector<bRGBA>::iterator iterator;
	typedef std::vector<bRGBA>::const_iterator const_iterator;

	Bitmap() { clear(); }
	explicit Bitmap(const std::string& filename) { clear(); load(filename); }
	Bitmap(unsigned int w, unsigned int h) { clear(); resize(w, h); }

	bool empty() const { return data.empty(); }
	void resize(unsigned int w, unsigned int h) { _width=w; _height=h; data.resize(_width*_height); }
	void clear() { _width=0; _height=0; data.clear(); }
	iterator begin() { return data.begin(); }
	iterator end()   { return data.end(); }
	const_iterator begin() const { return data.begin(); }
	const_iterator end()   const { return data.end(); }

	// bmp[y][x] 
	bRGBA* operator [] (int index) { return &data[_width*index]; }
	const bRGBA* operator [] (int index) const { return &data[_width*index]; }

	bool load(const std::string& filename);
	bool save(const std::string& filename) const;

	bool loadBMP(const std::string& filename);
	bool loadTGA(const std::string& filename);
	bool loadPNG(const std::string& filename);
	bool saveBMP(const std::string& filename) const;
	bool saveTGA(const std::string& filename) const;
	bool savePNG(const std::string& filename) const;

	unsigned int getWidth() const { return _width; }
	unsigned int getHeight() const { return _height; }
};


} //ist

#endif
