#include "dso.h"

#ifdef _WIN32
	#include <windows.h>
#endif
#ifdef linux
	#include <dlfcn.h>
	#include <unistd.h>
#endif



namespace ist {

#ifdef _WIN32

DynamicSharedObject::DynamicSharedObject() :
	handle(NULL)
{}

DynamicSharedObject::DynamicSharedObject(const std::string& filename) :
	handle(NULL)
{
	load(filename);
}

DynamicSharedObject::~DynamicSharedObject()
{
	unload();
}

bool DynamicSharedObject::load(const std::string& _filename)
{
	unload();
	filename = _filename;
	handle = ::LoadLibrary(filename.c_str());
	if(!(*this))
		filename.clear();
	return !!(*this);
}

void DynamicSharedObject::unload()
{
	if(!!(*this)) {
		::FreeLibrary(handle);
		filename.clear();
		handle = NULL;
	}
}

void* DynamicSharedObject::getFunction(const std::string& funcname) const
{
	if(!(*this))
		return 0;
#ifdef __MINGW32__
	return (void*)::GetProcAddress(handle, funcname.c_str());
#else
	return ::GetProcAddress(handle, funcname.c_str());
#endif
}

bool DynamicSharedObject::operator!() const
{
	return handle==NULL;
}

#endif
#ifdef linux

DynamicSharedObject::DynamicSharedObject() :
	handle(0)
{}

DynamicSharedObject::DynamicSharedObject(const std::string& path) :
	handle(0)
{
	load(path);
}

DynamicSharedObject::~DynamicSharedObject()
{
	unload();
}

bool DynamicSharedObject::load(const std::string& path)
{
	unload();

	if(path[0]!='/')
		filename = std::string(::get_current_dir_name())+'/'+path;
	else
		filename = path;

	handle = ::dlopen(filename.c_str(), RTLD_NOW | RTLD_GLOBAL);

	if(!(*this))
		filename.clear();
	return !!(*this);
}

void DynamicSharedObject::unload()
{
	if(!!(*this)) {
		::dlclose(handle);
		filename.clear();
		handle = 0;
	}
}

void* DynamicSharedObject::getFunction(const std::string& funcname) const
{
	if(!(*this))
		return 0;
	return ::dlsym(handle, funcname.c_str());
}

bool DynamicSharedObject::operator!() const
{
	return handle==0;
}

#endif

} // ist

