#ifndef GL_TEXTURE_INCLUDE
#define GL_TEXTURE_INCLUDE

#include <vector>
#include "i3d_texture.h"


namespace ist {
namespace i3d {


class Texture : public ITexture
{
private:
	Texture(const Texture&);
	Texture& operator =(const Texture&);
	void setDefaultParam();
	void setParam(unsigned int) const;
	void generate(unsigned int&, const Bitmap&);

	int _dimension;
	int _type;
	int _warp_s;
	int _warp_t;
	int _mag_filter;
	int _min_filter;
	int _env_mode;

	std::vector<unsigned int> _tex_name;

public:
	Texture();
	Texture(const std::string& filename, unsigned int cell_width=0, unsigned int cell_height=0);
	Texture(const Bitmap& src, unsigned int cell_width=0, unsigned int cell_height=0);
	~Texture();

	bool load(const std::string& filename, unsigned int cell_width=0, unsigned int cell_height=0);
	bool load(const Bitmap& src, unsigned int cell_width=0, unsigned int cell_height=0);
	void clear();
	void assign(int index=0) const;
	bool operator !()  const { return _tex_name.empty(); }

	unsigned int getName(int index=0) { return _tex_name[index]; }
	unsigned int getNumCell() const{ return _tex_name.size(); }
	void setDimension(int a) { _dimension=a; }
	void setType(int a)      { _type=a; }
	void setMagFilter(int a) { _mag_filter=a; }
	void setMinFilter(int a) { _min_filter=a; }
	void setWarpS(int a)     { _warp_s=a; }
	void setWarpT(int a)     { _warp_t=a; }
	void setEnvMode(int a)   { _env_mode=a; }

};


} //i3d
} //ist

#endif
