#ifndef ISTL_LUA_INCLUDE
#define ISTL_LUA_INCLUDE


extern "C" {
	#include <lua.h>
	#include <lualib.h>
	#include <lauxlib.h>
}
#include <string>
#include "exception.h"
#include "vector.h"
#include "matrix.h"


namespace ist {
namespace lua {


struct Thread
{
public:
	Thread()
	{ lstate=lua_open(); }

	~Thread() {
		if(lstate!=0)
			lua_close(lstate);
	}

	operator lua_State*() { return lstate; }
	lua_State* get() {return lstate; }
	void reset() {
		if(lstate!=0)
			lua_close(lstate);
		lstate=lua_open();
	}

private:
	Thread(const Thread&);
	Thread& operator =(const Thread&);

	lua_State *lstate;
};


namespace math3d {
	class StackSaver {
	public:
		StackSaver();
		~StackSaver();
	};

	void push_stack();
	void pop_stack();
}

void throw_error(lua_State *L, const char *message=0);

void math3dlibopen(lua_State *lstate);
void gllibopen(lua_State *lstate);

bool isvector2(lua_State *L, int index);
bool isvector3(lua_State *L, int index);
bool isvector4(lua_State *L, int index);
bool ismatrix22(lua_State *L, int index);
bool ismatrix33(lua_State *L, int index);
bool ismatrix44(lua_State *L, int index);

void pushvector2(lua_State *L, const vector2 &d);
void pushvector3(lua_State *L, const vector3 &d);
void pushvector4(lua_State *L, const vector4 &d);
void pushmatrix22(lua_State *L, const matrix22 &d);
void pushmatrix33(lua_State *L, const matrix33 &d);
void pushmatrix44(lua_State *L, const matrix44 &d);

vector2& tovector2(lua_State *L, int index);
vector3& tovector3(lua_State *L, int index);
vector4& tovector4(lua_State *L, int index);
matrix22& tomatrix22(lua_State *L, int index);
matrix33& tomatrix33(lua_State *L, int index);
matrix44& tomatrix44(lua_State *L, int index);


} // lua
} // ist



#endif
